 /*
	Cross-platform WMP Calling Functions - Macintosh Code
	Copyright  2007 Harry Whitfield
	
	CD Functions for Mac
	Copyright 2007 Michael Tughan

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as
	published by the Free Software Foundation; either version 2 of
	the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public
	License along with this program; if not, write to the Free
	Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
	MA  02110-1301  USA
	
	Cross-platform WMP Calling Functions version 2.8
	3 July, 2007
	Copyright  2007 Harry Whitfield
	mailto:g6auc@arrl.net
	
	Playlist Object and CD Functions
	28 May, 2007
	Copyright 2007 Michael Tughan
	mailto: michaelsprogramming@ca.inter.net
*/

// To play Windows Media using QuickTime Player on the Macintosh, you must
// also install the free Flip4Mac WMV Player, available from Microsoft.
// http://www.microsoft.com/windows/windowsmedia/player/wmcomponents.mspx

var printCount = 10;	// print limit

var TIMEOUT = 10;		// appleScript timeout

function WMP(url)	// Constructor of WMP object
{
	if (!url) { print('WMP: url: "' + url + '"'); }
	this.URL = url;
	this.autostart = false;
	var script  = 'close every movie\nset play movie from beginning when opened to false\n';
	if (url) { script += 'open location "' + this.URL + '"\n'; }
	WMP.doScript(script, 10, "");
}

WMP.prototype.quit = function() { WMP.doScript('quit\n', TIMEOUT, ""); };

WMP.doScript = function(script, timeout, defaultValue)
{
    if (printCount > 0) { print('script:\n' + script); printCount -= 1; }
    var result = appleScript('tell application "QuickTime Player"\n' + script + 'end tell\n', timeout);
    if (result.indexOf("AppleScript Error") >= 0) { print(result); return defaultValue; } return result;
};

// constants as used on Windows

WMP.wmppsUndefined     =  0;	// Windows Media Player is in an undefined state.
WMP.wmppsStopped       =  1;	// Playback of the current media item is stopped.
WMP.wmppsPaused        =  2;	// Playback of the current media item is paused. When a media item is paused, resuming playback begins from the same location.
WMP.wmppsPlaying       =  3;	// The current media item is playing.
WMP.wmppsScanForward   =  4;	// The current media item is fast forwarding.
WMP.wmppsScanReverse   =  5;	// The current media item is fast rewinding.
WMP.wmppsBuffering     =  6;	// The current media item is getting additional data from the server.
WMP.wmppsWaiting       =  7;	// Connection is established, but the server is not sending data. Waiting for session to begin.
WMP.wmppsMediaEnded    =  8;	// Media item has completed playback.
WMP.wmppsTransitioning =  9;	// Preparing new media item.
WMP.wmppsReady         = 10;	// Ready to begin playing.
WMP.wmppsReconnecting  = 11;	// Reconnecting to stream.

WMP.prototype.setURL = function(url, legend)
{
	if (!url) { print('setURL: url: "' + url + '"'); if (legend) { print('called from ' + legend); } }
	this.URL = url;
	if (url) { WMP.doScript('close every movie\nopen location "' + this.URL + '"\n', 10, ""); }
};
WMP.prototype.versionInfo = function() { return WMP.doScript('get the QuickTime version as string\n', TIMEOUT, ""); };
WMP.prototype.close = function() { WMP.doScript('if not (exists movie 1) then return\nclose movie 1\n', 10, ""); };
WMP.prototype.playState = function()
{
	var isPlaying = (WMP.doScript('if not (exists movie 1) then return false as string\nget the playing of movie 1 as string\n', TIMEOUT, "false") == "true");
	var isAtEnd   = (WMP.doScript('if not (exists movie 1) then return false as string\nget (the current time of movie 1 = the duration of movie 1) as string\n', TIMEOUT, "false") == "true");
	if (isPlaying)    { return WMP.wmppsPlaying;    }
	else if (isAtEnd) { return WMP.wmppsMediaEnded; }
	else 			  { return WMP.wmppsPaused;     }
};

WMP.prototype.mediaDuration = function ()
{
	var script  = 'if not (exists movie 1) then return "360000"\n';	// returns 360000 (10 minutes) if there is no movie
		script += 'get the duration of movie 1 as string\n';
	return Math.floor(Number( WMP.doScript(script, TIMEOUT, "0"))	/ 600);	// 1.666667 msecs per unit time
};
WMP.prototype.mediaDurationString = function () { WMP.formatTime(this.mediaDuration()); };	// returns the media duration (in [hh:]mm:ss)


WMP.prototype.play  = function() { WMP.doScript('if not (exists movie 1) then return\nplay movie 1\n', TIMEOUT, ""); };
WMP.prototype.pause = function() { WMP.doScript('if not (exists movie 1) then return\npause movie 1\n', TIMEOUT, ""); };

WMP.prototype.playPause = function() { if (this.playState() == WMP.wmppsPlaying) { this.pause(); } else { this.play(); } };

WMP.prototype.stop = function()  { WMP.doScript('if not (exists movie 1) then return\nstop movie 1\n', TIMEOUT, ""); };
WMP.prototype.fastForward = function()
{
	var script  = 'if not (exists movie 1) then return\nstop movie 1\n';
		script += 'set the current time of movie 1 to the duration of movie 1\n';
		WMP.doScript(script, TIMEOUT, "");
};
WMP.prototype.fastReverse = function() { WMP.doScript('if not (exists movie 1) then return\nstop movie 1\nrewind movie 1\n', TIMEOUT, ""); };

WMP.prototype.currentPosition = function ()
{
	var script  = 'if not (exists movie 1) then return "0"\n';
		script += 'get the current time of movie 1 as string\n';
	return Math.floor(Number( WMP.doScript(script, TIMEOUT, "0"))	/ 600);	// 1.666667 msecs per unit time
};

WMP.prototype.setCurrentPosition = function (seconds)
{
	WMP.doScript('if (exists movie 1) then set the current time of movie 1 to ' + Math.floor(600*seconds) + '\n', TIMEOUT, "");	// 1.666667 msecs per unit time
};

WMP.prototype.currentPositionString = function ()
{
	var script  = 'if not (exists movie 1) then return "0"\n';
		script += 'get the current time of movie 1 as string\n';
	return WMP.timeFromRunTime(Number(WMP.doScript(script, TIMEOUT, "0")));
};

WMP.prototype.getPlayerPosition = function ()
{
	var script  = 'if not (exists movie 1) then return "0"\n';
		script += 'get the current time of movie 1 as string\n';
	return WMP.timeFromRunTime(Number(WMP.doScript(script, TIMEOUT, "0")));
};

WMP.prototype.volume = function()
{ 
	var script  = 'set the current_volume to "0"\n';
		script += 'if (exists movie 1) then set the current_volume to (the sound volume of movie 1) as string\n';
		script += 'get the current_volume\n';
	return Math.round(WMP.doScript(script, TIMEOUT, "0") / 2.56 );
};
WMP.prototype.setVolume = function(volume)	// volume 0..100
{ 
	WMP.doScript('if (exists movie 1) then set the sound volume of movie 1 to ' + Math.round(2.56*volume) + '\n', TIMEOUT, "");
};
WMP.prototype.autoStart = function() { return this.autostart; };
WMP.prototype.setAutoStart = function(flag) 
{
	this.autostart = flag; var sFlag = "false"; if (flag) { sFlag = "true"; }
	WMP.doScript('set play movie from beginning when opened to ' + sFlag + '\n', TIMEOUT, "");
};

WMP.prototype.mute = function()
{
	var script  = 'set the muted_state to "false"\n';
		script += 'if (exists movie 1) then set the muted_state to (the muted of movie 1) as string\n';
		script += 'get the muted_state\n';
	return WMP.doScript(script, TIMEOUT, "false") == "true";
};

WMP.prototype.setMute = function(flag)
{
	var sFlag = "false"; if (flag) { sFlag = "true"; }
	WMP.doScript('if (exists movie 1) then set the muted of movie 1 to ' + sFlag + '\n', TIMEOUT, "");
};

WMP.prototype.bitRate = function()
{
	var script  = 'set the data_rate to "0"\n';
		script += 'if (exists movie 1) then set the data_rate to (the data rate of movie 1) as string\n';
		script += 'get the data_rate\n';
	return Math.round( WMP.doScript(script, TIMEOUT, "0") / 125);	// converted to kbps
};

WMP.prototype.downloadProgress = function()   { return 100; };	// not available

WMP.timeFromRunTime = function (time)
{
	function twoDigits(n) { n = String(n); while (n.length < 2) { n = "0" + n; } return n; }	

    var temp = Math.floor(time / 600);	// 1.666667 msecs per unit time
    var secs = temp % 60;
	temp = Math.round((temp - secs) / 60);
    var mins = temp % 60;
    var hours = Math.round((temp - mins) / 60);	
    
    var res = twoDigits(mins) + ':' + twoDigits(secs);
    if (hours !== 0) { res = String(hours) + ':' + res; }
    return res;
};

WMP.formatTime = function (time)    // convert time in seconds to [hh:]mm:ss
{
    function twoDigits(n) { n = String(n); while (n.length < 2) { n = "0" + n; } return n; }    

    time = Math.floor(time);
    var secs = time % 60;
    time = Math.round((time - secs) / 60);
    var mins = time % 60;
    var hours = Math.round((time - mins) / 60); 
    
    var res = twoDigits(mins) + ':' + twoDigits(secs);
    if (hours !== 0) { res = String(hours) + ':' + res; }
    return res;
};

WMP.prototype.setMiniaturized = function(flag)
{
    var sFlag = "false"; if (flag) { sFlag = "true"; }
    WMP.doScript('if (exists movie 1) then set the miniaturized of window 1 to ' + sFlag + '\n', TIMEOUT, "");
};

WMP.prototype.getCDs = function(sep) {
	if (sep === undefined) { sep = "|"; }
	var vols = filesystem.volumes, cds = [];
	for (var i = 0; i < vols.length; i += 1) {
		if ((vols[i].freeBytes === 0) && (filesystem.getDirectoryContents(vols[i].path)[0].indexOf(".TOC.plist") >= 0)) {
			cds[cds.length] = i + sep + filesystem.getDisplayName(vols[i].path);
		}
	}
	return cds;
};

WMP.prototype.getCD = function(index) {
	var path = filesystem.volumes[index].path;	// for example "/Volumes/Dolly Parton"
	var dir = filesystem.getDirectoryContents(path), playlist = new Playlist("Unknown Artist", path.substring(9, path.length));
	for (var i = 0; i < dir.length; i += 1) {
		if (dir[i] != ".TOC.plist") { playlist.addItem(dir[i].substring(dir[i].indexOf(" ") + 1, dir[i].length - 5), "file://" + path + "/" + dir[i]); }
	}
	return playlist;
};
